﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using PaymentEngine;

namespace Payment_Engine_Demo_App
{
    public partial class Process : Form
    {
        public Device device;
        public PaymentRequest request;

        public Process(Device dev)
        {
            InitializeComponent();
            btnRetry.Visible = false;
            OK_Button.Visible = false;
            device = dev;
        }

        public void startTransaction(TransactionRequest tran)
        {
            request = device.StartTransaction(tran,
                (status, result) =>
                {
                    Console.WriteLine("Status: " + status);

                    setButtonVisible(OK_Button, true);
                    setButtonVisible(Cancel_Button, false);

                    if(result == null)
                    {
                        setHeading("Transaction Failed");
                        setStatus(status);
                        return;
                    }

                    if (result != null)
                    {
                        setHeading("Transaction "  + result.Result);
                        
                        if (result.ResultCode.Equals("A"))
                        {
                            setStatus("RefNum " + result.RefNum + "  AuthCode: " + result.AuthCode);

                        } else {
                            setStatus("Error: " + result.Error);
                        }
                    }
                },
                (status) =>
                {
                    setStatus(status);
                    Console.WriteLine("Status: " + status);
                });

        }

        private void setHeading(String text)
        {
            if (lblHeading.InvokeRequired)
            {
                lblHeading.Invoke(new MethodInvoker(delegate { lblHeading.Text = text; }));
            }
            else
            {
                lblHeading.Text = text;
            }
        }

        private void setStatus(String status)
        {
            if (lblStatus.InvokeRequired)
            {
                lblStatus.Invoke(new MethodInvoker(delegate { lblStatus.Text = status; }));
            }
            else
            {
                lblStatus.Text = status;
            }
        }

        private void setButtonVisible(Button btn, bool visible)
        {

            if (btn.InvokeRequired)
            {
                btn.Invoke(new MethodInvoker(delegate { btn.Visible = visible; }));
            }
            else
            {
                btn.Visible = visible;
            }
        }


        private void OK_Button_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void Cancel_Button_Click(object sender, EventArgs e)
        {
            // request not running yet
            if(request==null) this.Close();

            // cancel transaction
            try
            {
                setStatus("Cancelling transaction...");
                request.CancelTransaction((x) =>
                {
                    setStatus("Transaction cancelled");
                });
            }
            catch (MiddlewareException ex)
            {
                Console.WriteLine("Exception cancelling tran: " + ex.Message);

            } finally
            {
                this.Close();

            }

        }

    }
}
